/* $Id: grass-defs.h,v 1.2 1994/07/05 14:32:53 ma Exp $ */

/*****************************************************************************/
/* Prototypes for most GRASS functions for use with ANSI C or C++	     */
/* Extracted from GRASS code (4.1.x) by Martin Ameskamp      		     */
/* (ma@informatik.uni-kiel.d400.de)					     */
/* I fixed some minor inconsistencies, but I'm afraid I forgot which :-(     */
/*****************************************************************************/

#ifndef GRASS_H
#define GRASS_H

#include <stdio.h>	/* for FILE	*/

struct MON_CAP;

#include <gis.h>

/*****************************************************************************/
/* GIS Library Prototypes - supported                                        */
/*****************************************************************************/
extern CELL * G_allocate_cell_buf (void);
extern CELL G_get_histogram_cat (int n, struct Histogram *histogram);
extern CELL G_number_of_cats (char *name, char *mapset);
extern FILE * G_fopen_append (char *element, char *name);
extern FILE * G_fopen_modify (char *element, char *name);
extern FILE * G_fopen_new (char *element, char *name);
extern FILE * G_fopen_old (char *element, char *name, char *mapset);
extern FILE * G_fopen_sites_new (char *name);
extern FILE * G_fopen_sites_old (char *name, char *mapset);
extern FILE * G_fopen_support_new (char *element, char *name, char *mapset);
extern FILE * G_fopen_support_old (char *element, char *name, char *mapset);
extern FILE * G_popen(char *cmd, char *mode);
extern int G_add_aspect_colors (struct Colors *colors, CELL min, CELL max);
extern int G_add_color_rule (CELL cat1, int r1, int g1, int b1, CELL cat2, int r2, int g2, int b2, struct Colors *colors);
extern int G_add_grey_scale_colors (struct Colors *colors, CELL min, CELL max);
extern int G_add_gyr_colors (struct Colors *colors, CELL min, CELL max);
extern int G_add_histogram (CELL cat, long int count, struct Histogram *histogram);
extern int G_add_modular_color_rule (CELL cat1, int r1, int g1, int b1, CELL cat2, int r2, int g2, int b2, struct Colors *colors);
extern int G_add_rainbow_colors (struct Colors *colors, CELL min, CELL max);
extern int G_add_ramp_colors  (struct Colors *colors, CELL min, CELL max);
extern int G_add_ryg_colors (struct Colors *colors, CELL min, CELL max);
extern int G_add_wave_colors  (struct Colors *colors, CELL min, CELL max);
extern int G_adjust_window_to_box (struct Cell_head *src, struct Cell_head *dst, int rows, int cols);
extern int G_ascii_check(char *string);
extern int G_begin_cell_area_calculations(void);
extern int G_begin_distance_calculations(void);
extern int G_begin_ellipsoid_polygon_area (double a, double e2);
extern int G_begin_geodesic_distance (double a, double e2);
extern int G_begin_geodesic_equation (double lon1, double lat1, double lon2, double lat2);
extern int G_begin_polygon_area_calculations(void);
extern int G_begin_rhumbline_equation (double lon1, double lat1, double lon2, double lat2);
extern int G_begin_zone_area_on_sphere (double r, double s);
extern int G_cellvalue_format (CELL v);
extern int G_clear_screen(void);
extern int G_close_cell (int fd);
extern int G_color_values (char *name, float *r, float *g, float *b);
extern int G_copy (char *a, char *b, int n);
extern int G_create_key_value(void);
extern int G_def_window(struct Cell_head *window);
extern int G_disable_interactive(void);
extern int G_done_msg(char *msg);
extern int G_edit_cats (char *name, struct Categories *cats, int option);
extern int G_edit_cellhd (struct Cell_head *cellhd, int type);
extern int G_edit_history (struct History *phist);
extern int G_extend_histogram (CELL cat, long int count, struct Histogram *histogram);
extern int G_fatal_error (char *msg);
extern int G_find_cell_stat (CELL cat, long int *count, struct Cell_stats *s);
extern int G_fork(void);
extern int G_format_easting (double east, char *buf, int projection);
extern int G_format_northing (double north, char *buf, int projection);
extern int G_format_resolution (double res, char *buf, int projection);
extern int G_free_cats (struct Categories *pcats);
extern int G_free_cell_stats (struct Cell_stats *s);
extern int G_free_colors (struct Colors *colors);
extern int G_free_fmatrix(float **m);
extern int G_free_histogram (struct Histogram *histogram);
extern int G_free_imatrix(int **m);
extern int G_free_ivector(int *v);
extern int G_free_key_value(struct Key_Value *kv);
extern int G_free_matrix(double **m);
extern int G_free_reclass (struct Reclass *reclass);
extern int G_free_tokens (char **tokens);
extern int G_free_vector(double *v);
extern int G_fwrite_key_value (FILE *fd, struct Key_Value *kv);
extern int G_get_cellhd (char *name, char *mapset, struct Cell_head *cellhd);
extern int G_get_color (CELL n, int *red, int *grn, int *blu, struct Colors *colors);
extern int G_get_color_range(CELL *min, CELL *max, struct Colors *colors);
extern int G_get_default_window (struct Cell_head *window);
extern int G_get_histogram_num (struct Histogram *histogram);
extern int G_get_map_row (int fd, CELL *buf, int row);
extern int G_get_map_row_nomask (int fd, CELL *cell, int row);
extern int G_get_range_min_max(struct Range *range, CELL *min, CELL *max);
extern int G_get_reclass (char *name, char *mapset, struct Reclass *reclass);
extern int G_get_set_window (struct Cell_head *window);
extern int G_get_site (FILE *fd, double *east, double *north, char **desc);
extern int G_get_window (struct Cell_head *window);
extern int G_getl (char *buf, int n, FILE *fd);
extern int G_gets (char *buf);
extern int G_gishelp(char *helpfile, char *request);
extern int G_gisinit(char *pgm);
extern int G_histogram_eq (struct Histogram *histo, unsigned char **map, CELL *min, CELL *max);
extern int G_init_cats (CELL num, char *title, struct Categories *pcats);
extern int G_init_cell_stats(struct Cell_stats *s);
extern int G_init_colors (struct Colors *colors);
extern int G_init_histogram (struct Histogram *histogram);
extern int G_init_range (struct Range *range);
extern int G_insert_commas(char *buf);
extern int G_invert_colors (struct Colors *colors);
extern int G_is_reclass (char *name, char *mapset, char *rname, char *rmapset);
extern int G_lat_format (double lat, char *buf);
extern int G_lat_parts (double lat, int *d, int *m, double *s, char *h);
extern int G_lat_scan (char *buf, double *lat);
extern int G_legal_filename (char *s);
extern int G_limit_east (double *east, int proj);
extern int G_limit_north (double *north, int proj);
extern int G_limit_south (double *south, int proj);
extern int G_limit_west (double *west, int proj);
extern int G_llres_format (double res, char *buf);
extern int G_llres_scan (char *buf, double *res);
extern int G_lon_format (double lon, char *buf);
extern int G_lon_parts (double lon, int *d, int *m, double *s, char *h);
extern int G_lon_scan (char *buf, double *lon);
extern int G_lookup_colors (CELL *cell, unsigned char *red, unsigned char *grn, unsigned char *blu, unsigned char *set, int n, struct Colors *colors);
extern int G_lookup_key_value_from_file(char *file, char *key, char *value, int n);
extern int G_ludcmp(double **a, int n, int *indx, double *d);
extern int G_make_aspect_colors (struct Colors *colors, CELL min, CELL max);
extern int G_make_colors (char *name, char *mapset, struct Colors *pcolr);
extern int G_make_grey_scale_colors (struct Colors *colors, CELL min, CELL max);
extern int G_make_gyr_colors (struct Colors *colors, CELL min, CELL max);
extern int G_make_histogram_cs (struct Cell_stats *statf, struct Histogram *histogram);
extern int G_make_histogram_eq_colors (struct Colors *colors, struct Cell_stats *statf);
extern int G_make_rainbow_colors (struct Colors *colors, CELL min, CELL max);
extern int G_make_ramp_colors  (struct Colors *colors, CELL min, CELL max);
extern int G_make_random_colors (struct Colors *colors, CELL min, CELL max);
extern int G_make_ryg_colors (struct Colors *colors, CELL min, CELL max);
extern int G_make_wave_colors  (struct Colors *colors, CELL min, CELL max);
extern int G_maskfd (void);
extern int G_next_cell_stat (CELL *cat, long int *count, struct Cell_stats *s);
extern int G_no_gisinit(void);
extern int G_open_cell_new (char *name);
extern int G_open_cell_new_random (char *name);
extern int G_open_cell_new_uncompressed (char *name);
extern int G_open_cell_old (char *name, char *mapset);
extern int G_open_new (char *element, char *name);
extern int G_open_old (char *element, char *name, char *mapset);
extern int G_open_support_new (char *element, char *name, char *mapset);
extern int G_open_support_old (char *element, char *name, char *mapset);
extern int G_open_update (char *element, char *name);
extern int G_parser(int argc, char **argv);
extern int G_pclose(FILE *ptr);
extern int G_percent (int n, int d, int s);
extern int G_plot_line (double east1, double north1, double east2, double north2);
extern int G_plot_line2 (double east1, double north1, double east2, double north2);
extern int G_plot_point (double east, double north);
extern int G_plot_polygon (double *x, double *y, int n);
extern int G_plot_where_en (int x, int y, double *east, double *north);
extern int G_plot_where_xy (double east, double north, int *x, int *y);
extern int G_program_name(void);
extern int G_projection (void);
extern int G_put_cell_title (char *name, char *title);
extern int G_put_cellhd (char *name, struct Cell_head *cellhd);
extern int G_put_map_row (int fd, CELL *buf);
extern int G_put_map_row_random (int fd, CELL *buf, int row, int col, int n);
extern int G_put_reclass (char *name, struct Reclass *reclass);
extern int G_put_site (FILE *fd, double east, double north, char *desc);
extern int G_put_window (struct Cell_head *window);
extern int G_putenv (char *name, char *value);
extern int G_read_cats (char *name, char *mapset, struct Categories *pcats);
extern int G_read_colors (char *name, char *mapset, struct Colors *colors);
extern int G_read_histogram (char *name, char *mapset, struct Histogram *histogram);
extern int G_read_history (char *name, char *mapset, struct History *hist);
extern int G_read_range (char *name, char *mapset, struct Range *range);
extern int G_read_vector_cats (char *name, char *mapset, struct Categories *pcats);
extern int G_remove (char *element, char *name);
extern int G_remove_colr (char *name);
extern int G_remove_commas(char *buf);
extern int G_rename (char *element, char *oldname, char *newname);
extern int G_rewind_cell_stats (struct Cell_stats *s);
extern int G_row_repeat_nomask (int fd, int row);
extern int G_row_update_range (CELL *cell, int n, struct Range *range);
extern int G_scan_easting (char *buf, double *easting, int projection);
extern int G_scan_northing (char *buf, double *northing, int projection);
extern int G_scan_resolution (char *buf, double *res, int projection);
extern int G_set_cat (CELL num, char *label, struct Categories *pcats);
extern int G_set_cats_fmt (char *fmt, float m1, float a1, float m2, float a2, struct Categories *pcats);
extern int G_set_cats_title (char *title, struct Categories *pcats);
extern int G_set_cell_format (int n);
extern int G_set_color (CELL cat, int r, int g, int b, struct Colors *colors);
extern int G_set_color_range (CELL min, CELL max, struct Colors *colors);
extern int G_set_distance_to_line_tolerance (double t);
extern int G_set_error_routine(int (*error_routine)(const char*, int));
extern int G_set_geodesic_distance_lat1(double lat1);
extern int G_set_geodesic_distance_lat2(double lat2);
extern int G_set_histogram (CELL cat, long int count, struct Histogram *histogram);
extern int G_set_key_value (char *key, char *value, struct Key_Value *kv);
extern int G_set_list_hit_return(int flag);
extern int G_set_program_name (char *s);
extern int G_set_window (struct Cell_head *window);
extern int G_setenv (char *name, char *value);
extern int G_setup_plot (double t, double b, double l, double r, int (*Move)(), int (*Cont)());
extern int G_shift_colors (int shift, struct Colors *colors);
extern int G_short_history (char *name, char *type, struct History *hist);
extern int G_shortest_way (double *east1, double *east2);
extern int G_sleep_on_error (int flag);
extern int G_sort_cats (struct Categories *pcats);
extern int G_sort_histogram (struct Histogram *histogram);
extern int G_sort_histogram_by_count (struct Histogram *histogram);
extern int G_strcasecmp(char *x, char *y);
extern int G_strip (char *buf);
extern int G_suppress_masking(void);
extern int G_suppress_warnings (int flag);
extern int G_svbksb(double **u, double *w, double **v, int m, int n, double *b, double *x);
extern int G_svdcmp(double **a, int m, int n, double *w, double **v);
extern int G_svelim(double *w, int n);
extern int G_system (char *command);
extern int G_tqli(double *d, double *e, int n, double **z);
extern int G_trim_decimal (char *buf);
extern int G_unopen_cell (int fd);
extern int G_unset_error_routine (void);
extern int G_unsetenv (char *name);
extern int G_update_cell_stats (CELL *cell, int n, struct Cell_stats *s);
extern int G_update_key_value_file (char *file, char *key, char *value);
extern int G_update_range (CELL cat, struct Range *range);
extern int G_usage(void);
extern int G_want_histogram(int flag);
extern int G_warning (char *msg);
extern int G_window_rows (void);
extern int G_window_cols (void);
extern int G_write_cats (char *name, struct Categories *cats);
extern int G_write_colors (char *name, char *mapset, struct Colors *colors);
extern int G_write_histogram (char *name, struct Histogram *histogram);
extern int G_write_histogram_cs (char *name, struct Cell_stats *statf);
extern int G_write_history (char *name, struct History *hist);
extern int G_write_key_value_file (char *file, struct Key_Value *kv, int *stat);
extern int G_write_range (char *name, struct Range *range);
extern int G_write_vector_cats (char *name, struct Categories *cats);
extern int G_write_zeros(int fd, long int n);
extern int G_yes (char *question, int dflt);
extern int G_zero (char *buf, int i);
extern int G_zero_cell_buf (CELL *buf);
extern int G_zero_histogram (struct Histogram *histogram);
extern int G_zone (void);
extern char * G_adjust_Cell_head (struct Cell_head *cellhd, int row_flag, int col_flag);
extern char * G_ask_any (char *prompt, char *name, char *element, char *desc, int warn);
extern char * G_ask_cell_any (char *prompt, char *name);
extern char * G_ask_cell_in_mapset (char *prompt, char *name);
extern char * G_ask_cell_new (char *prompt, char *name);
extern char * G_ask_cell_old (char *prompt, char *name);
extern char * G_ask_in_mapset (char *prompt, char *name, char *element, char *desc);
extern char * G_ask_new (char *prompt, char *name, char *element, char *desc);
extern char * G_ask_old (char *prompt, char *name, char *element, char *desc);
extern char * G_ask_sites_any (char *prompt, char *name);
extern char * G_ask_sites_in_mapset (char *prompt, char *name);
extern char * G_ask_sites_new (char *prompt, char *name);
extern char * G_ask_sites_old (char *prompt, char *name);
extern char * G_ask_vector_any (char *prompt, char *name);
extern char * G_ask_vector_in_mapset (char *prompt, char *name);
extern char * G_ask_vector_new (char *prompt, char *name);
extern char * G_ask_vector_old (char *prompt, char *name);
extern char * G_calloc (int m, int n);
extern char * G_chrcat (char *T, char *F, int n);
extern char * G_chrcpy (char *T, char *F, int n);
extern char * G_chrmov (char *T, char *F, int n);
extern char * G_color_name (int n);
extern char * G_database_projection_name();
extern char * G_date(void);
extern char * G_find_cell (char *name, char *mapset);
extern char * G_find_cell2 (char *name, char *mapset);
extern char * G_find_file (char *element, char *name, char *mapset);
extern char * G_find_file2 (char *element, char *name, char *mapset);
extern char * G_find_key_value (char *key, struct Key_Value *kv);
extern char * G_find_sites (char *name, char *mapset);
extern char * G_find_sites2 (char *name, char *mapset);
extern char * G_find_vector (char *name, char *mapset);
extern char * G_find_vector2 (char *name, char *mapset);
extern char * G_fully_qualified_name (char *name, char *mapset);
extern char * G_get_ask_return_msg(void);
extern char * G_get_cat (CELL num, struct Categories *pcats);
extern char * G_get_cats_title (struct Categories *pcats);
extern char * G_get_cell_title (char *name, char *mapset);
extern char * G_getenv (char *name);
extern char * G_gisbase(void);
extern char * G_gisdbase(void);
extern char * G_home (void);
extern char * G_index (char *str, char delim);
extern char * G_lat_format_string(void);
extern char * G_llres_format_string(void);
extern char * G_location(void);
extern char * G_location_path(void);
extern char * G_lon_format_string(void);
extern char * G_malloc (int n);
extern char * G_mapset(void);
extern char * G_mask_info (void);
extern char * G_myname(void);
extern char * G_realloc (char *buf, int n);
extern char * G_recreate_command(void);
extern char * G_rindex (char *str, char delim);
extern char * G_set_ask_return_msg (char *msg);
extern char * G_squeeze (char *line);
extern char * G_store (char *s);
extern char * G_strcat (char *T, char *F);
extern char * G_strcpy (char *T, char *F);
extern char * G_strdup(char *string);
extern char * G_strmov (char *T, char *F);
extern char * G_strncpy (char *T, char *F, int n);
extern char * G_strstr(char *mainString, char *subString);
extern char * G_tempfile(void);
extern char * G_tolcase (char *string);
extern char * G_toucase (char *string);
extern char * G_unctrl(unsigned char c);
extern char * G_whoami(void);
extern char ** G_tokenize (char *buf, char *delim);
extern char G_intr_char();
extern double * G_alloc_vector(int n);
extern double ** G_alloc_matrix(int rows, int cols);
extern double G_adjust_east_longitude (double east, double west);
extern double G_adjust_easting (double east, struct Cell_head *window);
extern double G_adjust_easting();
extern double G_area_for_zone_on_sphere (double north, double south);
extern double G_area_of_cell_at_row (int row);
extern double G_area_of_polygon(double *x, double *y, int n);
extern double G_col_to_easting (double col, struct Cell_head *window);
extern double G_darea0_on_ellipsoid();
extern double G_darea0_on_sphere (double lat);
extern double G_darea0_on_sphere();
extern double G_database_units_to_meters_fact();
extern double G_database_units_to_meters_fact();
extern double G_distance (double e1, double n1, double e2, double n2);
extern double G_distance2_point_to_line (double x, double y, double x1, double y1, double x2, double y2);
extern double G_easting_to_col (double east, struct Cell_head *window);
extern double G_ellipsoid_grid_dist (double a, double e2, double lon1, double lat1, double lon2, double lat2);
extern double G_ellipsoid_polygon_area (double *lon, double *lat, int n);
extern double G_geodesic_distance (double lon1, double lat1, double lon2, double lat2);
extern double G_geodesic_distance_lon_to_lon (double lon1, double lon2);
extern double G_geodesic_lat_from_lon (double lon);
extern double G_meridional_radius_of_curvature (double lon, double a, double e2);
extern double G_northing_to_row (double north, struct Cell_head *window);
extern double G_planimetric_polygon_area(double *x, double *y, int n);
extern double G_radius_of_conformal_tangent_sphere (double lon, double a, double e2);
extern double G_rhumbline_lat_from_lon (double lon);
extern double G_row_to_northing (double row, struct Cell_head *window);
extern double G_transverse_radius_of_curvature (double lon, double a, double e2);
extern float * G_alloc_fvector(int n);
extern float ** G_alloc_fmatrix(int rows, int cols);
extern int * G_alloc_ivector(int n);
extern int ** G_alloc_imatrix(int rows, int cols);
extern long G_get_histogram_count (int n, struct Histogram *histogram);
extern struct Flag * G_define_flag(void);
extern struct Key_Value * G_fread_key_value (FILE *fd);
extern struct Key_Value * G_get_projinfo() ;
extern struct Key_Value * G_get_projunits() ;
extern struct Key_Value * G_read_key_value_file(char *file, int *stat);
extern struct Option * G_define_option(void);
extern void G_lubksb(double **a, int n, int *indx, double *b);
extern void G_tred2(double **a, int n, double *d, double *e);

/*****************************************************************************/
/* GIS Library Prototypes - unsupported                                      */
/*****************************************************************************/
extern CELL G__read_cats (char *element, char *name, char *mapset, struct Categories *pcats, int full);
extern int G__check_for_auto_masking (void);
extern int G__check_format (int fd);
extern int G__check_gisinit(void);
extern int G__color_free_lookup (struct _Color_Info_ *cp);
extern int G__color_free_rules (struct _Color_Info_ *cp);
extern int G__color_reset (struct Colors *colors);
extern int G__create_alt_env(void);
extern int G__create_alt_search_path(void);
extern int G__create_window_mapping (int fd);
extern int G__init_window (void);
extern int G__insert_color_into_lookup (CELL cat, int red, int grn, int blu, struct _Color_Info_ *cp);
extern int G__interpolate_color_rule (CELL cat, unsigned char *red, unsigned char *grn, unsigned char *blu, struct _Color_Rule_ *rule);
extern int G__lookup_colors (CELL *cell, unsigned char *red, unsigned char *grn, unsigned char *blu, unsigned char *set, int n, struct Colors *colors, int mod, int rules_only);
extern int G__make_mapset_element (char *p_element);
extern int G__mapset_permissions (char *mapset);
extern int G__mask_info (char *name, char *mapset);
extern int G__name_in_mapset (char *name_in, char *name_out, char *mapset);
extern int G__name_is_fully_qualified (char *fullname, char *name, char *mapset);
extern int G__open (char *element, char *name, char *mapset, int mode);
extern int G__open_cell_new (char *name, int open_mode);
extern int G__open_cell_old (char *name, char *mapset);
extern int G__organize_colors (struct Colors *colors);
extern int G__projection_units(int n);
extern int G__put_window (struct Cell_head *window, char *dir, char *name);
extern int G__read_env(void);
extern int G__read_row_ptrs (int fd);
extern int G__set_gisrc_file(char *name);
extern int G__setenv (char *name, char *value);
extern int G__switch_env(void);
extern int G__switch_search_path(void);
extern int G__temp_element(char *element);
extern int G__write_Cell_head (FILE *fd, struct Cell_head *cellhd, int is_cellhd);
extern int G__write_cats (char *element, char *name, struct Categories *cats);
extern int G__write_colors (FILE *fd, struct Colors *colors);
extern int G__write_env(void);
extern int G__write_row_ptrs (int fd);
extern char * G__env_name (int n);
extern char * G__file_name (char *path, char *element, char *name, char *mapset);
extern char * G__get_gisrc_file(void);
extern char * G__get_window (struct Cell_head *window, char *element, char *name, char *mapset);
extern char * G__getenv (char *name);
extern char * G__home (void);
extern char * G__location_path(void);
extern char * G__machine_name(void);
extern char * G__mapset_name (int n);
extern char * G__projection_name(int n);
extern char * G__read_Cell_head (FILE *fd, struct Cell_head *cellhd, int is_cellhd);
extern char * G__tempfile (int pid);
extern char * G__unit_name (int unit, int plural);
extern char *G__get_window(struct Cell_head *window, char *element, char *name, char *mapset);

/*****************************************************************************/
/* Vector Library Prototypes - supported                                     */
/*****************************************************************************/
extern char * Vect_support_name (void);
extern int V1_close (struct Map_info *Map);
extern int V1_open_new (struct Map_info *Map, char *name);
extern int V1_open_old (struct Map_info *Map, char *name, char *mapset);
extern int V1_read_line (struct Map_info *Map, struct line_pnts *Points, long int offset);
extern int V1_read_next_line (struct Map_info *Map, struct line_pnts *line_p);
extern int V1_rewind (struct Map_info *Map);
extern int V2_area_att (struct Map_info *map, int area);
extern int V2_close (struct Map_info *Map);
extern int V2_get_area (struct Map_info *map, int num, struct P_area **Area);
extern int V2_get_area_bbox (struct Map_info *map, int area, double *N, double *S, double *E, double *W);
extern int V2_get_line_bbox (struct Map_info *map, int line, double *N, double *S, double *E, double *W);
extern int V2_line_att (struct Map_info *map, int line);
extern int V2_num_areas (struct Map_info *map);
extern int V2_num_lines (struct Map_info *map);
extern int V2_open_old (struct Map_info *Map, char *name, char *mapset);
extern int V2_read_line (struct Map_info *Map, struct line_pnts *line_p, int line);
extern int V2_read_next_line (struct Map_info *Map, struct line_pnts *line_p);
extern int V2_rewind (struct Map_info *Map);
extern int Vect_P_init (char *name, char *mapset, struct Map_info *map);
extern int Vect_append_point (struct line_pnts *Points, double x, double y);
extern int Vect_close (struct Map_info *Map);
extern int Vect_copy_head_data (struct dig_head *from, struct dig_head *to);
extern int Vect_copy_pnts_to_xy (struct line_pnts *Points, double *x, double *y, int *n);
extern int Vect_copy_xy_to_pnts (struct line_pnts *Points, double *x, double *y, int n);
extern int Vect_destroy_line_struct (struct line_pnts *p);
extern int Vect_get_area_points (struct Map_info *Map, int area, struct line_pnts *BPoints);
extern int Vect_get_isle_points (struct Map_info *Map, int isle, struct line_pnts *BPoints);
extern int Vect_init (void);
extern int Vect_level (struct Map_info *Map);
extern int Vect_open_new (struct Map_info *Map, char *name);
extern int Vect_open_old (struct Map_info *Map, char *name, char *mapset);
extern int Vect_print_header (struct Map_info *Map);
extern int Vect_read_next_line (struct Map_info *Map, struct line_pnts *line_p);
extern int Vect_remove_constraints (struct Map_info *Map);
extern int Vect_reset_line (struct line_pnts *Points);
extern int Vect_rewind (struct Map_info *Map);
extern int Vect_set_constraint_region (struct Map_info *Map, double n, double s, double e, double w);
extern int Vect_set_constraint_type (struct Map_info *Map, int type);
extern int Vect_set_open_level (int level);
extern int Vect_x_read_head_binary (struct Map_info *Map, struct dig_head *dhead);
extern int Vect_x_write_head_binary (struct Map_info *Map, struct dig_head *dhead);
extern long Vect_write_line (struct Map_info *Map, int type, struct line_pnts *points);
extern struct line_pnts * Vect_new_line_struct (void);

/*****************************************************************************/
/* Vector Library Prototypes - not supported                                 */
/*****************************************************************************/
extern int V1__open_update_1 (struct Map_info *Map, char *name);
extern int V2__init_for_create_plus (struct Map_info *Map, char *name);
extern int V2__open_new_1 (struct Map_info *Map, char *name);
extern int V2__open_update_1 (struct Map_info *Map, char *name);
extern int V2__release_file_mem (struct Map_info *Map);
extern int V2__setup_for_digit (struct Map_info *Map, char *name);
extern int V__map_overlap (struct Map_info *Map, double n, double s, double e, double w);
extern int Vect__P_writeable (int x);
extern int Vect__Read_line (struct Map_info *Map, struct line_pnts *p, long int offset);
extern int Vect__Rewrite_line (struct Map_info *Map, long int offset, char type, struct line_pnts *points);
extern int Vect__copy_portable_info (struct dig_head *from, struct dig_head *to);
extern int Vect__get_window (struct Map_info *Map, double *n, double *s, double *e, double *w);
extern int Vect__init_head (struct dig_head *head);
extern int Vect__open_update_1 (struct Map_info *Map, char *name);
extern int Vect__read_head_binary (struct Map_info *Map, struct dig_head *head);
extern int Vect__write_head_binary (struct Map_info *Map, struct dig_head *head);
extern int Vect_x__Mem_Read_line (struct line_pnts *p, long int offset);
extern int Vect_x__Read_line (struct Map_info *Map, struct line_pnts *p, long int offset);
extern int Vect_x__Rewrite_line (struct Map_info *Map, long int offset, char type, struct line_pnts *points);
extern long Vect__Write_line (struct Map_info *Map, char type, struct line_pnts *points);
extern long Vect_x__Write_line(struct Map_info *Map, char type, struct line_pnts *points);
extern struct dig_head * Vect__get_default_in_head (void);
extern struct dig_head * Vect__get_default_out_head (void);
extern struct dig_head * Vect__get_default_port_head (void);
extern struct line_pnts * Vect__new_line_struct (void);

/*****************************************************************************/
/* Raster Library Prototypes                                                 */
/*****************************************************************************/
extern int R_box_abs(int x1, int y1, int x2, int y2);
extern int R_box_rel(int x, int y);
extern int R_color_table_float(void);
extern int R_color_table_fixed(void);
extern int R_color_offset(int n);
extern int R_color(int index);
extern int R_standard_color(int index);
extern int R_cont_abs(int x, int y);
extern int R_cont_rel(int x, int y);
extern int R_erase(void);
extern int R_flush(void);
extern int R_font(char *name);
extern int R_get_num_colors (int *n);
extern int R_get_text_box(char *sometext, int *t, int *b, int *l, int *r);
extern int R_get_location_with_box(int cx, int cy, int *wx, int *wy, int *button);
extern int R_get_location_with_line(int cx, int cy, int *wx, int *wy, int *button);
extern int R_get_location_with_pointer(int *wx, int *wy, int *button);
extern int R_move_abs(int x, int y);
extern int R_move_rel(int x, int y);
extern int R_panel_save(char *name, int t, int b, int l, int r);
extern int R_panel_restore(char *name);
extern int R_panel_delete(char *name);
extern int R_polydots_abs(int *xarray, int *yarray, int number);
extern int R_polydots_rel(int *xarray, int *yarray, int number);
extern int R_polyline_abs(int *xarray, int *yarray, int number);
extern int R_polyline_rel(int *xarray, int *yarray, int number);
extern int R_polygon_abs(int *xarray, int *yarray, int number);
extern int R_polygon_rel(int *xarray, int *yarray, int number);
extern int R_RGB_color(unsigned char red, unsigned char grn, unsigned char blu);
extern int R_set_RGB_color(unsigned char *r, unsigned char *g, unsigned char *b);
extern int R_RGB_raster(int n, int nrows, unsigned char *red, unsigned char *grn, unsigned char *blu, int withzeros);
extern int R_raster(int num, int nrows, int withzero, int *ras);
extern int R_raster_char(int num, int nrows, int withzero, char *ras);
extern int R_raster_int(int num, int nrows, int withzero, int *ras);
extern int R_reset_color(unsigned char red, unsigned char grn, unsigned char blu, int number);
extern int R_reset_colors(int min, int max, unsigned char *red, unsigned char *grn, unsigned char *blu);
extern int R_screen_left(void);
extern int R_screen_rite(void);
extern int R_screen_bot(void);
extern int R_screen_top(void);
extern int R_set_window(int t, int b, int l, int r);
extern int R_text(char *sometext);
extern int R_text_rotation(float rotation);
extern int R_text_size(int width, int height);
extern int R_open_driver(void);
extern int R_close_driver(void);
extern int R_release_driver(void);
extern int R_stabilize(void);
extern int R_pad_append_item (char *item, char *value);
extern int R_pad_delete_item (char *name);
extern int R_pad_get_item (char *item, char ***list, int *count);
extern int R_pad_list_items (char ***list, int *count);
extern int R_pad_set_item (char *item, char *value);
extern int R_pad_freelist (char **list, int count);
extern int R_pad_create (char *pad);
extern int R_pad_current (char *name);
extern int R_pad_delete (void);
extern int R_pad_invent (char *pad);
extern int R_pad_list (char ***list, int *count);
extern int R_pad_select (char *pad);
extern struct MON_CAP * R_parse_monitorcap(int field, char *key);
extern int R_pad_perror (char *msg, int code);

/*****************************************************************************/
/* Display Library Prototypes                                                */
/*****************************************************************************/
extern int D_clip(double s, double n, double w, double e, double *x, double *y, double *c_x, double *c_y);
extern int D_do_conversions(struct Cell_head *window, int t, int b, int l, int r);
extern double D_get_u_to_a_yconv(void);
extern double D_get_u_to_a_xconv(void);
extern double D_get_a_to_d_xconv(void);
extern double D_get_a_to_d_yconv(void);
extern double D_get_u_to_d_xconv(void);
extern double D_get_u_to_d_yconv(void);
extern double D_get_u_west(void);
extern double D_get_u_east(void);
extern double D_get_u_north(void);
extern double D_get_u_south(void);
extern double D_get_a_west(void);
extern double D_get_a_east(void);
extern double D_get_a_north(void);
extern double D_get_a_south(void);
extern double D_get_d_west(void);
extern double D_get_d_east(void);
extern double D_get_d_north(void);
extern double D_get_d_south(void);
extern double D_u_to_a_row(double U_row);
extern double D_u_to_a_col(double U_col);
extern double D_a_to_d_row(double A_row);
extern double D_a_to_d_col(double A_col);
extern double D_u_to_d_row(double U_row);
extern double D_u_to_d_col(double U_col);
extern double D_d_to_u_row(double D_row);
extern double D_d_to_u_col(double D_col);
extern double D_d_to_a_row(double D_row);
extern double D_d_to_a_col(double D_col);
extern double D_get_ns_resolution(void);
extern double D_get_ew_resolution(void);
extern char * D_color_list();
extern double D_get_d_south();
extern double D_get_d_north();
extern double D_get_d_east();
extern double D_get_d_west();
extern int D_set_clip_window (int Top, int Bottom, int Left, int Right);
extern int D_set_clip_window_to_map_window (void);
extern int D_cont_abs (int x, int y);
extern int D_cont_rel (int x, int y);
extern int D_move_abs (int x, int y);
extern int D_move_rel (int x, int y);
extern int D_set_cell_name(char *name);
extern int D_get_cell_name(char *name);
extern int D_add_to_list(char *string);
extern int D_clear_window(void);
extern int D_claim_offset_is(int num);
extern int D_offset_is(int *num);
extern int D_set_erase_color(char *colorname);
extern int D_get_erase_color(char *colorname);
extern int D_popup(int back_colr, int text_colr, int div_colr, int top, int left, int percent_per_line, char **options);
extern int D_draw_cell(int A_row, CELL *xarray, struct Colors *colors);
extern int D_cell_draw_setup(int t, int b, int l, int r);
extern int D_set_overlay_mode (int n);
extern int D_set_colors (struct Colors *colors);
extern int D_reset_color (CELL cat, int r, int g, int b);
extern int D_color (CELL cat, struct Colors *colors);
extern int D_lookup_colors (CELL *raster, int ncols, struct Colors *colors);
extern int D_raster (CELL *raster, int ncols, int nrows, struct Colors *colors);
extern int D_check_colormap_size (CELL min, CELL max, int *ncolors);
extern int D_scan_double (char *buf, double *f);
extern int D_scan_float (char *buf, float *f);
extern int D_scan_int (char *buf, int *f);
extern int D_setup (int clear);
extern int D_translate_color(char *str);
extern int D_new_window(char *name, int t, int b, int l, int r);
extern int D_set_cur_wind(char *name);
extern int D_get_cur_wind(char *name);
extern int D_show_window(int color);
extern int D_get_screen_window(int *t, int *b, int *l, int *r);
extern int D_check_map_window(struct Cell_head *wind);
extern int D_reset_screen_window(int t, int b, int l, int r);
extern int D_timestamp(void);
extern int D_remove_window(void);
extern int D_erase_window(void);

#endif /* GRASS_H */
